using System;
using System.Xml.XPath;
using System.Xml;
using System.Collections;

namespace Multiconn.Xml
{
	/// <summary>	
	/// IndexingXPathNavigator enables lazy or eager indexing of any XML store
	/// (XmlDocument, XPathDocument or any other IXPathNavigable store) thus
	/// providing alternative way to select nodes directly from built index table 
	/// and not searhing the tree. This allows drastically decrease selection time
	/// on preindexed selections.
	/// </summary>
	/// <remarks> Oleg Tkachenko, oleg@tkachenko.com</remarks>
	public class IndexingXPathNavigator : XPathNavigator
	{
    private XPathNavigator nav;
    private XPathNavigatorIndexManager manager;    

    /// <summary>
    /// Creates IndexingXPathNavigator over specified XPathNavigator.
    /// </summary>
    /// <param name="nav">Core XPathNavigator</param>
		public IndexingXPathNavigator(XPathNavigator nav)
		{
			this.nav = nav;      
      manager = new XPathNavigatorIndexManager();
		} 
   
    /// <summary>
    /// Builds indexes according to defined keys.
    /// </summary>
    public void BuildIndexes() 
    {
      manager.BuildIndexes();
    }

    /// <summary>
    /// Adds named key for use with key() function.
    /// </summary>
    /// <param name="keyname">The name of the key</param>
    /// <param name="match">XPath pattern, defining the nodes to which 
    /// this key is applicable</param>
    /// <param name="use">XPath expression used to determine 
    /// the value of the key for each matching node</param>
    public virtual void AddKey(string keyname, string match, string use) 
    {      
      KeyDef key = new KeyDef(nav, match, use);      
      manager.AddKey(nav, keyname, key);
    }
    
    /// <summary>
    /// Compiles XPath expressions using base XPathNavigator.Compile()
    /// method and set IndexingXsltContext instance to the result of 
    /// the compilation so compiled expressions support key() extension
    /// function.
    /// </summary>
    /// <param name="xpath">XPath expression to complile</param>
    /// <returns>Compiled XPath expression with augmented context 
    /// to support key() extension function</returns>
    public override XPathExpression Compile(string xpath) 
    {
      XPathExpression expr = base.Compile(xpath);       
      expr.SetContext(new IndexingXsltContext(manager, nav.NameTable));
      return expr;
    }

    /// <summary>
    /// Selects a node set using the specified XPath expression.
    /// </summary>
    /// <param name="xpath">A string representing an XPath expression</param>
    /// <returns>An XPathNodeIterator pointing to the selected node set</returns>
    public override XPathNodeIterator Select(string xpath)
    {
      XPathExpression expr = Compile(xpath);
      return base.Select(expr);
    }

    /// <summary>
    /// Creates new cloned version of the IndexingXPathNavigator.
    /// </summary>
    /// <returns>Cloned copy of the IndexingXPathNavigator</returns>
    public override XPathNavigator Clone() 
    {
      return new IndexingXPathNavigator(nav.Clone());
    }

    #region XPathNavigator dummy overrides    
		
    public override XPathNodeType NodeType 
    { 
      get { return nav.NodeType; }
    }

    public override string LocalName 
    {         
      get { return nav.LocalName; }
    }

    public override string Name 
    { 
      get { return nav.Name; }
    }
		
    public override string NamespaceURI 
    {
      get { return nav.NamespaceURI; }
    }
      
    public override string Prefix 
    {
      get { return nav.Prefix; }
    }

    public override string Value 
    {
      get { return nav.Value; }
    }

    public override String BaseURI 
    {
      get { return nav.BaseURI; } 
    }

    public override bool IsEmptyElement 
    {
      get { return nav.IsEmptyElement; }
    }
      
    public override string XmlLang 
    {
      get { return nav.XmlLang; }
    }
      
    public override XmlNameTable NameTable 
    {
      get { return nav.NameTable; }
    }		

    public override bool HasAttributes 
    {
      get { return nav.HasAttributes; }			
    }

    public override string GetAttribute(string localName, string namespaceURI) 
    {
      return nav.GetAttribute(localName, namespaceURI);
    }

    public override bool MoveToAttribute(string localName, string namespaceURI) 
    {
      return nav.MoveToAttribute(localName, namespaceURI);
    }

    public override bool MoveToFirstAttribute() 
    {
      return nav.MoveToFirstAttribute();
    }

    public override bool MoveToNextAttribute() 
    {
      return nav.MoveToNextAttribute();
    }
		
    public override string GetNamespace(string localname) 
    {
      return nav.GetNamespace(localname);
    }
		
    public override bool MoveToNamespace(string @namespace) 
    {
      return nav.MoveToNamespace(@namespace);
    }

    public override bool MoveToFirstNamespace(XPathNamespaceScope namespaceScope) 
    {
      return nav.MoveToFirstNamespace(namespaceScope);
    }

    public override bool MoveToNextNamespace(XPathNamespaceScope namespaceScope) 
    {
      return nav.MoveToNextNamespace(namespaceScope);
    }       		
    
    public override bool MoveToNext() 
    {		    
      return nav.MoveToNext();			    
    }
        
    public override bool MoveToPrevious() 
    {		    
      return nav.MoveToPrevious();
    }

    public override bool MoveToFirst() 
    {		    
      return nav.MoveToFirst();
    }
		
    public override bool MoveToFirstChild() 
    {      
      return nav.MoveToFirstChild();			
    }
    
    public override bool MoveToParent() 
    {      
      return nav.MoveToParent();      
    }

    public override void MoveToRoot() 
    {		    
      nav.MoveToRoot();
    }
		
    public override bool MoveTo(XPathNavigator other) 
    {
      return nav.MoveTo(other);
    }
		
    public override bool MoveToId(string id) 
    {            
      return nav.MoveToId(id);
    }

    public override bool IsSamePosition(XPathNavigator other) 
    {            
      return nav.IsSamePosition(other);
    }
    
    public override bool HasChildren 
    {
      get { return nav.HasChildren; }
    }
    #endregion
	}
}
