using System;
using System.Xml.XPath;

namespace Multiconn.Xml
{
	/// <summary>
	/// Compilable key definition.
	/// </summary>
  internal class KeyDef
  {
    private string match, use;
    private XPathExpression matchExpr, useExpr;
    private XPathNavigator nav;

    /// <summary>
    /// Creates a key definition with specified 'match' and 'use' expressions.
    /// </summary>
    /// <param name="nav">XPathNavigator to compile XPath expressions</param>
    /// <param name="match">XPath pattern, defining the nodes to 
    /// which this key is applicable</param>
    /// <param name="use">XPath expression expression used to 
    /// determine the value of the key for each matching node.</param>
    public KeyDef(XPathNavigator nav, string match, string use)
    {
      this.nav= nav;
      this.match = match;
      this.use = use;
    }

    /// <summary>
    /// XPath pattern, defining the nodes to 
    /// which this key is applicable.
    /// </summary>
    public string Match 
    {
      get { return match; }
      set { match = value; }
    }

    /// <summary>
    /// XPath expression expression used to 
    /// determine the value of the key for each matching node.
    /// </summary>
    public string Use 
    {
      get { return use; }
      set { use = value; }
    }  

    /// <summary>
    /// Compiled XPath pattern, defining the nodes to 
    /// which this key is applicable.
    /// </summary>
    public XPathExpression MatchExpr 
    {
      get { 
        if (matchExpr == null)
          matchExpr = nav.Compile(match);
        return matchExpr; 
      }      
    }

    /// <summary>
    /// Compiled XPath expression expression used to 
    /// determine the value of the key for each matching node.
    /// </summary>
    public XPathExpression UseExpr 
    {
      get { 
        if (useExpr == null)
          useExpr = nav.Compile(use);
        return useExpr; 
      }      
    }  

    /// <summary>
    /// Compiles match and use expressions.
    /// </summary>
    public void Compile() 
    {
      matchExpr = nav.Compile(match);
      useExpr = nav.Compile(use);
    }    
  }
}
