using System;
using System.Collections;
using System.Xml.XPath;
using System.Reflection;

namespace Multiconn.Xml
{
	/// <summary>
	/// Index table for XPathNavigator.
	/// </summary>
  internal class XPathNavigatorIndex 
  {
    private ArrayList keys;
    private Hashtable index;
    private Type arrayIteratorType;
    
    /// <summary>
    /// Creates index over specified XPathNavigator.
    /// </summary>
    /// <param name="nav"></param>
    public XPathNavigatorIndex() 
    {
      keys = new ArrayList();  
      index = new Hashtable();      
      Assembly systemXml = typeof(XPathNodeIterator).Assembly; 
      arrayIteratorType = systemXml.GetType("System.Xml.XPath.XPathArrayIterator");
    }
    
    /// <summary>
    /// Adds a key.
    /// </summary>
    /// <param name="key">Key definition</param>
    public void AddKey(KeyDef key) 
    {      
      keys.Add(key);
    }
      
    /// <summary>
    /// Returns indexed nodes by a key value.
    /// </summary>
    /// <param name="keyValue">Key value</param>    
    public XPathNodeIterator GetNodes(object keyValue) 
    {          
      //As per XSLT spec:
      //When the second argument to the key function is of type node-set, 
      //then the result is the union of the result of applying the key function 
      //to the string value of each of the nodes in the argument node-set. 
      //When the second argument to key is of any other type, the argument is 
      //converted to a string as if by a call to the string function; it 
      //returns a node-set containing the nodes in the same document as 
      //the context node that have a value for the named key equal to this string.      
      ArrayList indexedNodes = null;
      if (keyValue is XPathNodeIterator) 
      {
        XPathNodeIterator nodes = keyValue as XPathNodeIterator;
        while (nodes.MoveNext()) 
        {        
          ArrayList chunkOfIndexedNodes = (ArrayList)index[nodes.Current.Value];          
          if (chunkOfIndexedNodes != null) 
          {
            if (indexedNodes == null)
              indexedNodes = new ArrayList();
            indexedNodes.AddRange(chunkOfIndexedNodes);        
          }
        }
      } 
      else 
      {        
        indexedNodes = (ArrayList)index[keyValue.ToString()];
      }      
      if (indexedNodes == null)
        indexedNodes = new ArrayList(0);            

      return (XPathNodeIterator) Activator.CreateInstance( arrayIteratorType, 
        BindingFlags.Instance | BindingFlags.Public | 
        BindingFlags.CreateInstance, null, new object[]{indexedNodes}, null ); 
    }

    /// <summary>
    /// Matches given node against "match" pattern and adds it to 
    /// the index table if the matching succeeded.
    /// </summary>
    /// <param name="node">Node to match</param>
    public void MatchNode(XPathNavigator node) 
    {
      foreach (KeyDef keyDef in keys) 
      {
        if (node.Matches(keyDef.MatchExpr)) 
        {
          //Ok, let's calculate key value(s). As per XSLT spec:
          //If the result is a node-set, then for each node in the node-set, 
          //the node that matches the pattern has a key of the specified name whose 
          //value is the string-value of the node in the node-set; otherwise, the result 
          //is converted to a string, and the node that matches the pattern has a 
          //key of the specified name with value equal to that string.        
          object key = node.Evaluate(keyDef.UseExpr);        
          if (key is XPathNodeIterator) 
          { 
            XPathNodeIterator ni = (XPathNodeIterator)key;
            while (ni.MoveNext())          
              AddNodeToIndex(node, ni.Current.Value);
          }         
          else
          {
            AddNodeToIndex(node, key.ToString());
          }        
        }
      }
    }

    /// <summary>
    /// Adds node to the index slot according to key value.
    /// </summary>
    /// <param name="node">Node to add to index</param>
    /// <param name="key">String key value</param>
    private void AddNodeToIndex(XPathNavigator node, string key) 
    {
      //Get slot
      ArrayList indexedNodes = (ArrayList)index[key];
      if (indexedNodes == null) 
      {
        indexedNodes = new ArrayList();
        index.Add(key, indexedNodes);
      }
      indexedNodes.Add(node.Clone());
    }        
  }
}
